import React, { useState, useEffect } from 'react';
import {
    Table,
    TableBody,
    TableCell,
    TableContainer,
    TableHead,
    TableRow,
    Paper,
    Tooltip,
    TextField,
    Typography,
    Button,
    Box
} from '@mui/material';
import { SpreadsheetService } from '../../openapi';
import * as ExcelJS from 'exceljs';
import { saveAs } from 'file-saver';

const SpreadsheetTable = ({ raw_data, errors, headers, setRawData, onCancel, fileBlob }) => {
    const [localErrors, setLocalErrors] = useState(errors || []);
    const [editingCell, setEditingCell] = useState({});
    const [nonEditableCells, setNonEditableCells] = useState(new Set());

    const generateErrorMap = (errorsList) => {
        const errorMap = new Map();
        if (Array.isArray(errorsList)) {
            errorsList.forEach(error => {
                const key = `${error.row}-${headers[error.cell]}`;
                errorMap.set(key, error.message);
            });
        }
        return errorMap;
    };

    const errorMap = generateErrorMap(localErrors);

    useEffect(() => {
        const initialNonEditableCells = new Set();

        raw_data.forEach((row, rowIndex) => {
            headers.forEach((_, colIndex) => {
                const key = `${row.row_num}-${headers[colIndex]}`;
                if (!errorMap.has(key)) {
                    initialNonEditableCells.add(`${rowIndex}-${colIndex}`);
                }
            });
        });

        setNonEditableCells(initialNonEditableCells);
    }, [raw_data, headers, errorMap]);

    const handleCellEdit = async (rowIndex, colIndex) => {
        const updatedRawData = [...raw_data];
        const columnName = headers[colIndex];
        const currentRow = updatedRawData[rowIndex];
        const newValue = editingCell[`${rowIndex}-${colIndex}`];

        if (newValue === undefined) return;

        if (!currentRow.data) {
            currentRow.data = [];
        }

        currentRow.data[colIndex] = newValue;

        setEditingCell(prev => {
            const updated = { ...prev };
            delete updated[`${rowIndex}-${colIndex}`];
            return updated;
        });

        try {
            const response = await SpreadsheetService.validateCellValidateCellPost({
                row: currentRow.row_num,
                column: columnName,
                value: newValue
            });

            if (response.is_valid !== undefined) {
                if (response.is_valid) {
                    const updatedErrors = localErrors.filter(
                        error => !(error.row === currentRow.row_num && error.cell === colIndex)
                    );
                    setLocalErrors(updatedErrors);
                    setNonEditableCells(prev => new Set([...prev, `${rowIndex}-${colIndex}`]));
                } else {
                    const updatedErrors = [
                        ...localErrors,
                        { row: currentRow.row_num, cell: colIndex, message: response.message || 'Invalid value.' }
                    ];
                    setLocalErrors(updatedErrors);
                }
            }
            setRawData(updatedRawData);
        } catch (error) {
            console.error('Validation failed:', error);
        }
    };

    const handleCellBlur = (rowIndex, colIndex) => {
        handleCellEdit(rowIndex, colIndex);
    };

    const allCellsValid = () => nonEditableCells.size === raw_data.length * headers.length;

    const handleSubmit = async () => {
        if (allCellsValid()) {
            console.log('All data is valid. Proceeding with submission...');
        } else {
            console.log('There are validation errors in the dataset. Please correct them before submission.');
        }
    };

    const downloadCorrectedSpreadsheet = async () => {
        const workbook = new ExcelJS.Workbook();
        await workbook.xlsx.load(fileBlob);
        const worksheet = workbook.getWorksheet(1);

        raw_data.forEach((row, rowIndex) => {
            row.data.forEach((value, colIndex) => {
                worksheet.getRow(row.row_num).getCell(colIndex + 1).value = value;
            });
        });

        const buffer = await workbook.xlsx.writeBuffer();
        const blob = new Blob([buffer], { type: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' });

        saveAs(blob, 'corrected_data.xlsx');
    };

    useEffect(() => {
        console.log('Raw data:', raw_data);
        console.log('Errors:', localErrors);
        console.log('Headers:', headers);
    }, [raw_data, localErrors, headers]);

    if (!raw_data || !headers) {
        return <div>Loading...</div>;
    }

    return (
        <TableContainer component={Paper}>
            <Table>
                <TableHead>
                    <TableRow>
                        {headers.map((header, index) => (
                            <TableCell key={index} align="center">
                                <Typography variant="body2">{header}</Typography>
                            </TableCell>
                        ))}
                    </TableRow>
                </TableHead>
                <Box display="flex" justifyContent="space-between" mt={2}>
                    <Button variant="contained" color="secondary" onClick={onCancel}>
                        Cancel
                    </Button>
                    <Button variant="contained" color="primary" onClick={handleSubmit} disabled={!allCellsValid()}>
                        Submit
                    </Button>
                    <Button variant="contained" onClick={downloadCorrectedSpreadsheet} disabled={!allCellsValid()}>
                        Download Corrected Spreadsheet
                    </Button>
                </Box>
                <TableBody>
                    {raw_data.map((row, rowIndex) => (
                        <TableRow key={rowIndex}>
                            {headers.map((header, colIndex) => {
                                const key = `${row.row_num}-${header}`;
                                const errorMessage = errorMap.get(key);
                                const isInvalid = !!errorMessage;
                                const cellValue = (row.data && row.data[colIndex]) || "";
                                const editingValue = editingCell[`${rowIndex}-${colIndex}`];
                                const isReadonly = !isInvalid;

                                return (
                                    <TableCell key={colIndex} align="center">
                                        <Tooltip title={errorMessage || ""} arrow disableHoverListener={!isInvalid}>
                                            <TextField
                                                value={editingValue !== undefined ? editingValue : cellValue}
                                                onChange={(e) => setEditingCell({ ...editingCell, [`${rowIndex}-${colIndex}`]: e.target.value })}
                                                onKeyDown={(e) => {
                                                    if (e.key === "Enter") {
                                                        handleCellEdit(rowIndex, colIndex);
                                                    }
                                                }}
                                                onBlur={() => handleCellBlur(rowIndex, colIndex)}
                                                error={isInvalid}
                                                fullWidth
                                                variant="outlined"
                                                size="small"
                                                disabled={isReadonly}
                                            />
                                        </Tooltip>
                                    </TableCell>
                                );
                            })}
                        </TableRow>
                    ))}
                </TableBody>
            </Table>
        </TableContainer>
    );
};

export default SpreadsheetTable;